@@ -6,35 +6,6 @@
 library('httr') # for access to the HTTP header
 library('rjson')
 
-# Customize httr content parsers
-#
-# Add csv support to httr and return a data frame for csv and json content
-#
-# @author Hugh J. Devlin \email{Hugh.Devlin@@cityofchicago.org}
-assignParsers <- function() {
-	
-	# Add CSV parser
-	assign("text/csv", 
-			function(x) {
-				read.csv(textConnection(x), stringsAsFactors=FALSE)
-			}, 
-			envir=httr:::parsers
-	)
-	
-	# Replace JSON parser
-	assign("application/json",
-			function(x) {
-				content = rawToChar(x)
-				if(content=="[ ]") # empty JSON?
-					data.frame() # empty data frame
-				else
-					data.frame(t(sapply(fromJSON(content), unlist)), stringsAsFactors=FALSE)
-			}, 
-			envir=httr:::parsers
-	)
-	
-}
-
 #' Time-stamped message
 #'
 #' Issue a time-stamped, origin-stamped log message. 
@@ -63,12 +34,17 @@ fieldName <- function(humanName) {
 
 #' Convert Socrata calendar_date string to POSIX
 #'
-#' @param x a string in Socrata calendar_date format
+#' @param x char in Socrata calendar_date format
 #' @return a POSIX date
 #' @export
 #' @author Hugh J. Devlin, Ph. D. \email{Hugh.Devlin@@cityofchicago.org}
 posixify <- function(x) {
-	strptime(as.character(x), format="%m/%d/%Y %I:%M:%S %p")
+	x <- as.character(x)
+	# Two calendar date formats supplied by Socrata
+	if(regexpr("^[[:digit:]]{2}/[[:digit:]]{2}/[[:digit:]]{4}$", x[1])[1] == 1) 
+		strptime(x, format="%m/%d/%Y")
+	else
+		strptime(x, format="%m/%d/%Y %I:%M:%S %p")
 }
 
 # Wrap httr GET in some diagnostics
@@ -78,19 +54,54 @@ posixify <- function(x) {
 # @param url Socrata Open Data Application Program Interface (SODA) query
 # @return httr response object
 # @author Hugh J. Devlin, Ph. D. \email{Hugh.Devlin@@cityofchicago.org}
-get <- function(url) {
+getResponse <- function(url) {
 	response <- GET(url)
 	status <- http_status(response)
 	if(response$status_code != 200) {
 		details <- content(response)
-		if(nrow(details) > 0) {
-			logMsg(paste("Error detail:", details$code[1], details$message[1]))
-		}
+		logMsg(paste("Error in httr GET:", details$code[1], details$message[1]))
 	}
 	stop_for_status(response)
 	response
 }
 
+# Content parsers
+#
+# Return a data frame for csv and json
+#
+# @author Hugh J. Devlin \email{Hugh.Devlin@@cityofchicago.org}
+# @param an httr response object
+# @return data frame, possibly empty
+getContentAsDataFrame <- function(response) { UseMethod('response') }
+getContentAsDataFrame <- function(response) {
+	mimeType <- response$header$'content-type'
+	# skip optional parameters
+	sep <- regexpr(';', mimeType)[1]
+	if(sep != -1) mimeType <- substr(mimeType, 0, sep[1] - 1)
+	switch(mimeType,
+		'text/csv' = 
+				read.csv(textConnection(content(response)), stringsAsFactors=FALSE),
+		'application/json' = 
+				if(content(response, as='text') == "[ ]") # empty json?
+					data.frame() # empty data frame
+				else
+					data.frame(t(sapply(fromJSON(rawToChar(content(response, as='raw'))), unlist)), stringsAsFactors=FALSE)
+	) # end switch
+}
+
+# Get the SoDA 2 data types
+#
+# Get the Socrata Open Data Application Program Interface data types from the http response header
+# @author Hugh J. Devlin, Ph. D. \email{Hugh.Devlin@@cityofchicago.org}
+# @param responseHeaders headers attribute from an httr response object
+# @return a named vector mapping field names to data types
+getSodaTypes <- function(response) { UseMethod('response') }
+getSodaTypes <- function(response) {
+	result <- fromJSON(response$headers[['x-soda2-types']])
+	names(result) <- fromJSON(response$headers[['x-soda2-fields']])
+	result
+}
+
 #' Get a full Socrata data set as an R data frame
 #'
 #' Manages throttling and POSIX date-time conversions
@@ -103,18 +114,20 @@ get <- function(url) {
 #' earthquakes <- read.socrata("http://soda.demo.socrata.com/resource/4tka-6guv.json")
 read.socrata <- function(url) {
 	url <- as.character(url)
-	assignParsers()
-	response <- get(url)
-	page <- content(response)
+	parsedUrl <- parse_url(url)
+	if(substr(parsedUrl$path, 1, 9) != 'resource/')
+		stop("Error in read.socrata: url ", url, " is not a Socrata SoDA resource.")
+	mimeType <- guess_media(parsedUrl$path)
+	if(mimeType != 'text/csv' && mimeType != 'application/json')
+		stop("Error in read.socrata: ", mimeType, " not a supported data format.")
+	response <- getResponse(url)
+	page <- getContentAsDataFrame(response)
 	result <- page
-	# create a named vector mapping field names to data types
-	dataTypes <- fromJSON(response$headers[['x-soda2-types']])
-	names(dataTypes) <- fromJSON(response$headers[['x-soda2-fields']])
+	dataTypes <- getSodaTypes(response)
 	while (nrow(page) > 0) { # more to come maybe?
-		query <- paste(url, if(regexpr("\\?", url)[1] == -1){'?'} else {"&"}, '$offset=', nrow(result), sep='')
-		response <- get(query)
-		stop_for_status(response)
-		page <- content(response)
+		query <- paste(url, if(is.null(parsedUrl$query)) {'?'} else {"&"}, '$offset=', nrow(result), sep='')
+		response <- getResponse(query)
+		page <- getContentAsDataFrame(response)
 		result <- rbind(result, page) # accumulate
 	}	
 	# convert Socrata calendar dates to posix format
